@@ -816,6 +816,14 @@ static char *k2pdfopt_options=
 "                  exactly match the font used by the document.  Use -ocrsp+\n"
 "                  to allow more than one space between each word in the row\n"
 "                  of text in order to optimize the selection position.\n"
+"-ocrvbb[-]        Verify OCR bounding boxes.  For PDF files that have a built-\n"
+"                  in OCR layer, if the resulting text selection does not seem to\n"
+"                  match the graphical word positions in the document, you can\n"
+"                  try this option.  It checks the bounding box for each word\n"
+"                  from the OCR layer against the actual graphical placement of\n"
+"                  the word in the document and \"shrinks\" the bounding box to\n"
+"                  just fit around the word.  This only affects non-native PDF\n"
+"                  conversions.  Default is -ocrvbb- (turned off).\n"
 "-ocrvis <s|t|b>   Set OCR visibility flags.  Put 's' to show the source doc,\n"
 "                  't' to show the OCR text, and/or 'b' to put a box around\n"
 "                  each word.  Default is -ocrvis s.  To show both the source\n"
