@@ -1,7 +1,31 @@
+// Copyright (C) 2004-2021 Artifex Software, Inc.
+//
+// This file is part of MuPDF.
+//
+// MuPDF is free software: you can redistribute it and/or modify it under the
+// terms of the GNU Affero General Public License as published by the Free
+// Software Foundation, either version 3 of the License, or (at your option)
+// any later version.
+//
+// MuPDF is distributed in the hope that it will be useful, but WITHOUT ANY
+// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+// FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more
+// details.
+//
+// You should have received a copy of the GNU Affero General Public License
+// along with MuPDF. If not, see <https://www.gnu.org/licenses/agpl-3.0.en.html>
+//
+// Alternative licensing terms are available from the licensor.
+// For commercial licensing, see <https://www.artifex.com/> or contact
+// Artifex Software, Inc., 39 Mesa Street, Suite 108A, San Francisco,
+// CA 94129, USA, for further information.
+
 #include "mupdf/fitz.h"
 
 #include "color-imp.h"
 
+#include <string.h>
+
 #if FZ_ENABLE_ICC
 
 #ifndef LCMS_USE_FLOAT
@@ -36,13 +60,30 @@ static void fz_premultiply_row(fz_context *ctx, int n, int c, int w, unsigned ch
 	for (; w > 0; w--)
 	{
 		a = s[n1];
-		for (k = 0; k < c; k++)
-			s[k] = fz_mul255(s[k], a);
+		if (a == 0)
+			memset(s, 0, c);
+		else if (a != 255)
+			for (k = 0; k < c; k++)
+				s[k] = fz_mul255(s[k], a);
 		s += n;
 	}
 }
 
-static void fz_unmultiply_row(fz_context *ctx, int n, int c, int w, unsigned char *s, const unsigned char *in)
+static void fz_premultiply_row_0or1(fz_context *ctx, int n, int c, int w, unsigned char *s)
+{
+	unsigned char a;
+	int n1 = n-1;
+	for (; w > 0; w--)
+	{
+		a = s[n1];
+		if (a == 0)
+			memset(s, 0, c);
+		s += n;
+	}
+}
+
+/* Returns 0 for all the alphas being 0, 1 for them being 0 or 255, 2 otherwise. */
+static int fz_unmultiply_row(fz_context *ctx, int n, int c, int w, unsigned char *s, const unsigned char *in)
 {
 	int a, inva;
 	int k;
@@ -50,15 +91,62 @@ static void fz_unmultiply_row(fz_context *ctx, int n, int c, int w, unsigned cha
 	for (; w > 0; w--)
 	{
 		a = in[n1];
-		inva = a ? 255 * 256 / a : 0;
+		if (a != 0)
+			goto nonzero;
 		for (k = 0; k < c; k++)
-			s[k] = (in[k] * inva) >> 8;
+			s[k] = 0;
 		for (;k < n1; k++)
 			s[k] = in[k];
-		s[n1] = a;
+		s[n1] = 0;
 		s += n;
 		in += n;
 	}
+	return 0;
+	for (; w > 0; w--)
+	{
+		a = in[n1];
+nonzero:
+		if (a != 0 && a != 255)
+			goto varying;
+		k = 0;
+		if (a == 0)
+			for (; k < c; k++)
+				s[k] = 0;
+		for (;k < n; k++)
+			s[k] = in[k];
+		s += n;
+		in += n;
+	}
+	return 1;
+	for (; w > 0; w--)
+	{
+		a = in[n1];
+varying:
+		if (a == 0)
+		{
+			for (k = 0; k < c; k++)
+				s[k] = 0;
+			for (;k < n1; k++)
+				s[k] = in[k];
+			s[k] = 0;
+		}
+		else if (a == 255)
+		{
+			memcpy(s, in, n);
+		}
+		else
+		{
+			inva = 255 * 256 / a;
+			for (k = 0; k < c; k++)
+				s[k] = (in[k] * inva) >> 8;
+			for (;k < n1; k++)
+				s[k] = in[k];
+			s[n1] = a;
+		}
+		s += n;
+		in += n;
+	}
+	return 2;
 }
 
 struct fz_icc_link
@@ -211,7 +299,8 @@ fz_new_icc_link(fz_context *ctx,
 	fz_colorspace *prf,
 	fz_color_params rend,
 	int format,
-	int copy_spots)
+	int copy_spots,
+	int premult)
 {
 	GLOINIT
 	cmsHPROFILE src_pro = src->u.icc.profile;
@@ -260,6 +349,11 @@ fz_new_icc_link(fz_context *ctx,
 	if (copy_spots)
 		flags |= cmsFLAGS_COPY_ALPHA;
 
+#ifdef cmsFLAGS_PREMULT
+	if (premult)
+		flags |= cmsFLAGS_PREMULT;
+#endif
+
 	if (prf_pro == NULL)
 	{
 		transform = cmsCreateTransform(GLO src_pro, src_fmt, dst_pro, dst_fmt, rend.ri, flags);
@@ -323,7 +417,7 @@ fz_new_icc_link(fz_context *ctx,
 	}
 	fz_catch(ctx)
 	{
-		cmsDeleteTransform(GLO link);
+		cmsDeleteTransform(GLO transform);
 		fz_rethrow(ctx);
 	}
 	return link;
@@ -359,7 +453,7 @@ fz_icc_transform_color(fz_context *ctx, fz_color_converter *cc, const float *src
 }
 
 void
-fz_icc_transform_pixmap(fz_context *ctx, fz_icc_link *link, fz_pixmap *src, fz_pixmap *dst, int copy_spots)
+fz_icc_transform_pixmap(fz_context *ctx, fz_icc_link *link, const fz_pixmap *src, fz_pixmap *dst, int copy_spots)
 {
 	GLOINIT
 	int cmm_num_src, cmm_num_dst, cmm_extras;
@@ -390,28 +484,60 @@ fz_icc_transform_pixmap(fz_context *ctx, fz_icc_link *link, fz_pixmap *src, fz_p
 
 	inputpos = src->samples;
 	outputpos = dst->samples;
+
+#ifdef cmsFLAGS_PREMULT
+	/* LCMS2MT can only handle premultiplied data if the number of 'extra'
+	 * channels is the same. If not, do it by steam. */
+	if (sa && cmm_extras != (int)T_EXTRA(dst_format))
+#else
+	/* Vanilla LCMS2 cannot handle premultiplied data. If present, do it by steam. */
 	if (sa)
+#endif
 	{
 		buffer = fz_malloc(ctx, ss);
 		for (; h > 0; h--)
 		{
-			fz_unmultiply_row(ctx, sn, sc, sw, buffer, inputpos);
-			cmsDoTransform(GLO link->handle, buffer, outputpos, sw);
-			fz_premultiply_row(ctx, dn, dc, dw, outputpos);
+			int mult = fz_unmultiply_row(ctx, sn, sc, sw, buffer, inputpos);
+			if (mult == 0)
+			{
+				/* Solid transparent row. No point in doing the transform
+				 * because it will premultiplied back to 0. */
+				memset(outputpos, 0, ds);
+			}
+			else
+			{
+				cmsDoTransform(GLO link->handle, buffer, outputpos, sw);
+				if (!copy_spots)
+				{
+					/* Copy the alpha by steam */
+					unsigned char *d = outputpos + dn - 1;
+					const unsigned char *s = inputpos + sn -1;
+					int w = sw;
+
+					while (w--)
+					{
+						*d = *s;
+						d += dn;
+						s += sn;
+					}
+				}
+				if (mult == 1)
+					fz_premultiply_row_0or1(ctx, dn, dc, dw, outputpos);
+				else if (mult == 2)
+					fz_premultiply_row(ctx, dn, dc, dw, outputpos);
+			}
 			inputpos += ss;
 			outputpos += ds;
 		}
 		fz_free(ctx, buffer);
 	}
 	else
-	{
 		for (; h > 0; h--)
 		{
 			cmsDoTransform(GLO link->handle, inputpos, outputpos, sw);
 			inputpos += ss;
 			outputpos += ds;
 		}
-	}
 }
 
 #endif
